<?php
/**
 * Copyright (©) 2020 Pinpoint Designs LTD. All right's reserved.
 *
 * @category    Pinpoint
 * @copyright   Copyright (c) 2020 Pinpoint Designs. (http://www.pinpointdesigns.co.uk)
 * @author      Daniel Cuevas <daniel@pinpointdesigns.co.uk>
 */

namespace Pinpoint\DropShipping\Controller\Adminhtml\Dropship;

use Magento\Backend\App\Action;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Message\ManagerInterface;
use Magento\Sales\Api\OrderItemRepositoryInterface;
use Magento\Sales\Model\OrderRepository;
use Pinpoint\DropShipping\Model\DropShipItemFactory;
use Pinpoint\DropShipping\Model\DropShipItemRepository;
use Pinpoint\DropShipping\Model\SupplierRepository;

/**
 * Class MassDropship
 */
class MassDropship extends Action
{
    const READY_FOR_DESPATCH = 'ready_for_despatch';
    const PREPARING_FOR_DESPATCH = 'preparing_for_despatch';

    /**
     * @var OrderItemRepositoryInterface
     */
    private $orderItemRepository;
    /**
     * @var DropShipItemFactory
     */
    private $dropShipItemFactory;
    /**
     * @var OrderRepository
     */
    private $orderRepository;
    /**
     * @var DropShipItemRepository
     */
    private $dropShipItemRepository;
    /**
     * @var SupplierRepository
     */
    private $supplierRepository;
    /**
     * @var \Magento\Sales\Api\Data\OrderItemInterface[]
     */
    private $_items;

    /**
     * MassDropship constructor.
     *
     * @param OrderItemRepositoryInterface $orderItemRepository
     * @param DropShipItemFactory $dropShipItemFactory
     * @param OrderRepository $orderRepository
     * @param ManagerInterface $messageManager
     */
    public function __construct(
        OrderItemRepositoryInterface $orderItemRepository,
        DropShipItemFactory $dropShipItemFactory,
        OrderRepository $orderRepository,
        DropShipItemRepository $dropShipItemRepository,
        SupplierRepository $supplierRepository,
        Action\Context $context
    )
    {
        parent::__construct($context);
        $this->orderItemRepository = $orderItemRepository;
        $this->dropShipItemFactory = $dropShipItemFactory;
        $this->orderRepository = $orderRepository;
        $this->dropShipItemRepository = $dropShipItemRepository;
        $this->supplierRepository = $supplierRepository;
    }

    /**
     * @inheritDoc
     */
    public function execute()
    {
        try {

            $orderId = $this->getRequest()->getParam('order_id');
            $itemIds = $this->getRequest()->getParam('item_id');
            $supplierIds = $this->getRequest()->getParam('supplier');

            $order = $this->orderRepository->get($orderId);
            if ($order->getStatus() !== self::READY_FOR_DESPATCH) {
                throw new LocalizedException(__('The order status must be "Being Prepared For Despatch - Non Cancellable"'));
            }

            $this->initializeOrderItems($order);

            $count = 0;

            foreach ($itemIds as $key => $itemId) {
                if (isset($supplierIds[$key])) {
                    $supplier = $this->supplierRepository->get($supplierIds[$key]);
//                    $orderItem = $this->orderItemRepository->get($itemId);
                    $orderItem = $this->getOrderItem($itemId);
                    $dropshipItem = $this->dropShipItemFactory->create();
                    $dropshipItem->setDropshippingData($orderItem, $supplier);
                    $orderItem->setSupplierName($supplier->getCode());
                    $orderItem->setMethod($supplier->getMethod());

                    $dropshipItem->save();
                    $count++;
                }
            }

            if ($count) {
                $this->messageManager->addSuccessMessage("Dropshipped {$count} item(s).");

                if ($count === count($itemIds)) {
                    $order->setStatus(self::PREPARING_FOR_DESPATCH);
                    $this->messageManager->addSuccessMessage('The order status is now "Preparing for Despatch"');
                }

                $this->orderRepository->save($order);
            }

        } catch (LocalizedException $e) {
            $this->messageManager->addErrorMessage($e->getMessage());
        } catch (\Exception $e) {
            $this->messageManager->addErrorMessage('There was an error when trying to complete the dropshipping process. Please try again.');
        }

       return $this->_redirect('sales/order/view', ['order_id' => $orderId]);
    }

    private function getOrderItem($itemId)
    {
        if (isset($this->_items[$itemId])) {
            return $this->_items[$itemId];
        }

        return null;
    }

    private function initializeOrderItems(\Magento\Sales\Api\Data\OrderInterface $order)
    {
        if (is_null($this->_items)) {
            $this->_items = [];

            foreach ($order->getItems() as $item) {
                $this->_items[$item->getItemId()] = $item;
            }
        }
    }
}
